; Adventure Vision MEGA bios for Code RED v1.07 by MrBlinky March 2013
;===============================================================================

;MEGA bios is a custom bios with special display capabilities and small size
;for use with the MEGA demo.
;Some parts are taken from or inspired by justburns micro and mini bios.

.ifndef	MEGABIOS_ASM
.set MEGABIOS_ASM, 1
	.ifndef MC8048_ASM
	.include "includes\mc8048.asm"
	.endif
	
;- hardware -

.equ	BIOS_OFF,	$04		;Disable BIOS

.equ	BUTTONSON,	$F8		;enable button pullups
.equ	XBUTTONSON,	$F8+BIOS_OFF	;enable button pullups with bios disabled

.equ	VRAM0,	$00		;External RAM bank 0
.equ	VRAM1,	$01		;External RAM bank 1
.equ	VRAM2,	$02		;External RAM bank 2
.equ	VRAM3,	$03		;External RAM bank 3

.equ	XVRAM0,	$00+BIOS_OFF	;External RAM bank 0 with bios disabled
.equ	XVRAM1,	$01+BIOS_OFF	;External RAM bank 1 with bios disabled
.equ	XVRAM2,	$02+BIOS_OFF	;External RAM bank 2 with bios disabled
.equ	XVRAM3,	$03+BIOS_OFF	;External RAM bank 3 with bios disabled

;- internal ram variables -

.ifndef	CODERED_ASM
.equ	IRAM_DSPOS,			$3C ;display position for transition
.equ	IRAM_DSPMODE,		$3D	;global display mode 
.equ	IRAM_FRAMES,		$3E	;Frame counter LSB       
.equ	IRAM_FRAMES_MSB,	$3F	;Frame counter MSB       

.endif

;- general constants -

.equ	FPS,		15		;frames per second
.equ	FRAME_TIME	$48888		;number of cycles in one display frame

;- display modes -

.equ	DM_GREY		$80		;gray color mode
.equ	DM_SCROLL	$40		;scroll display left out
.equ	DM_BLEED	$60		;bleeding left to right

;--------------------------------
;cold start entry point
;--------------------------------

	.org $000

;	call AV_SOUND		;kill any power up sound
	call AV_CLRVRAM		;clear all vram
	clr	a
	mov	psw,a			;clear flags and stack
	mov r0,IRAM_LAST	;clear all IRAM incl regs
clriram:
	mov @r0,a			;clear iram and regs
	djnz r0,clriram
	sel rb1			;use register bank 1 as normal registers
	jmp start

;clear all vram banks (screens)
;---------------------------------------	

;returns with VRAM3 selected

AV_CLRVRAM:
clrvram:

	mov	a,XVRAM2	;start in VRAM2
clrvrlp:	
	outl p1,a
	xch	a,r1
	call cls
	xch a,r1
	dec	a
	jb2	clrvrlp		;loop until VRAM0 done
	
	;coninue in AV_CLS_VRAM3 below
	
;clear and select VRAM3screen
;---------------------------------------	

AV_CLS_VRAM3:
	orl	p1,XVRAM3
	
;clear current selected screen
;---------------------------------------	

AV_CLS:
cls:	
	clr a
	mov r0,a
	dec a
clslp:
	movx @r0,a
	djnz r0,clslp
	ret
	
;--------------------------------
;update led bar
;--------------------------------

;takes 36 cycles including call

;changes:
;	a,r0

ledbar:				
	orl p2,$20		;2\ LED 0-7 storage register (port is normally 0 and will at end of display so this will work fine)
	movx a,@ r0		;2| Read data from VRAM to register
	mov a,$40		;2| LED 8-15 storage register
	outl p2,a		;2| select it
	inc r0			;1| Next VRAM location
	movx a,@ r0		;2| Read data from VRAM to register
	orl p2,$60		;2| LED 16-23 storage register
	inc r0			;1| Next VRAM location
	movx a,@ r0		;2| Read data from VRAM to register
	mov a,$80		;2| Select fourth LED storage register
	outl p2,a		;2| select it
	inc r0			;1| Next memory location
	movx a,@ r0		;2| Move data to LED storage register
	orl p2,$A0		;2|
	inc r0			;1| Next memory location
	movx a,@ r0		;2| Move data to LED storage register
	mov a,$10		;2| latch storage registers to LED outputs
	outl p2,a		;2| change all LED outputs synchronously
	ret				;2/=34 cycles	

;---------------------------------------
;Set frame trigger (byte)
;---------------------------------------

;set frame trigger to negetive value in a

;entry:
;	 a = negative frame count value

;exit:
;	none

;changes:
;	r0,r1

AV_SETTRIGGER1:
	mov	r1,$FF
	;jmp	AV_SETTRIGGER2
	
;---------------------------------------
;Set frame trigger (word)
;---------------------------------------

;set frame trigger to negetive value in r1.a

;entry:
;	 a = lsb negative frame count value
;	r1 = msb negative frame count value

;exit:
;	none

;changes:
;	r0

AV_SETTRIGGER2:	
	mov	r0,IRAM_FRAMES
	mov	@r0,a
	inc	r0
	xch	a,r1
	mov	@r0,a
	xch	a,r1
	ret

;---------------------------------------
;show and wait for two seconds
;---------------------------------------

;entry:
;	none

;exit:
;	none

;changes:
;   a,r0,r1,r2,r3

AV_SHOWWAIT2:
	mov	a,-2*FPS
	
;---------------------------------------
;show and wait
;---------------------------------------

;same as show but waits for -a frames

;entry:
;	 a = negative frames to wait

;exit:
;	none

;changes:
;   a,r0,r1,r2,r3
	
AV_SHOWWAIT:
	call	AV_SETTRIGGER1
av_swl:	
	call	AV_SHOW
	jc		av_swl
	ret

AV_SHOW_WITHSCROLL:	
	mov	r2,50				;50 frames for scroll out
	call AV_TESTTRIGGER1
	jnc dsp_noscroll
	mov	r1,IRAM_DSPMODE
	mov a,@r1
	orl	a,DM_SCROLL
	mov @r1,a
dsp_noscroll:
	jmp	AV_SHOW
	
;---------------------------------------
;show always VRAM3
;---------------------------------------

;entry:
;	none

;exit:
;	none

;changes:
;   a,r0,r1,r2,r3
	
AV_SHOW_VRAM3:
	orl	p1,XVRAM3
	
;---------------------------------------
;show current VRAM bank
;---------------------------------------

;display vram, play tracker tune and test for frame counter overflow

;entry:
;	none

;exit:
;	none

;changes:
;   a,r0,r1,r2,r3

AV_SHOW:
;custom display routines
;---------------------------------------

;displays currently selected screen

;AV_DISPLAY:
;display:
	sel	rb0
	mov	r1,IRAM_FRAMES	;update framecounter
	inc	@r1
	mov	a,@r1
	jnz		$+4
	inc	r1
	inc	@r1
	
	mov	r1,IRAM_DSPMODE 
	mov	a,@r1
	dec r1			;IRAM_DSPOS
	jb6	$+4	
	mov	@r1,6		;show full display
	mov r2,a		;save flag
	mov	a,@r1		;get left vram position to display
    mov r0,a		
	add a,5			;next frame show 1 pixels less of display
	mov @r1,a
	
	mov a,r2		;get flag
	cpl a			;test not bit
	mov	r2,0		;no bleeding value
	jb5 synchi		;no bleeding
	
	mov a,r0
	cpl a
	inc a
	mov r2,a		;negative bleeding vptr
	mov r0,6		;bleeding always full screen
	
synchi:
	jnt1 $			;wait for sync to end in case of multiple calls within same sync period
	jt1 $			;wait for sync low
	mov a,r0		;display vram address
	jz dsp_off
doledsb:
	anl p2,0
doleds:
	call ledbar			;36 copy VRAM to LED bar   

	mov r1,IRAM_DSPMODE	;2
	mov a,@r1			;1
	nop					;1
	jb7	greymode	;2 deal with grey display

	;42 cycles passed at this point
	
	mov	r1,41-2		;2\ 
ledbrl:
	djnz r1,ledbrl	;2
	inc r0			;1\ next display column
;	inc r0			;1  adjust for djnz
;	djnz r0,doleds	;2/
	
	mov a,r0		;1
	jz dsp_end		;2
	
	mov	a,r0		;1 smearing
	add	a,r2		;1    
	jnc	doleds		;2 display normally	

	;don't update LEDs for smearing fx
	
	mov	a,r0		;1
	add	a,5-1		;2 update Vptr
	mov	r0,a		;1
	
	mov	r1,39+15	;2
	jmp	ledbrl		;2
	
;	anl	p2,0		;enable display blanking
;	sel	rb1
;	ret	
	
	;grey mode 
	
greymode:
	mov	r1,19-4		;2
grybr1:	djnz	r1,grybr1	;2

	mov	a,r0		;1
	add	a,256-4		;2
	mov	r0,a		;1
	anl	p1,$FE		;2 select grey bank
	
	call ledbar		;36
	orl p1,$01		;2 select bright bank
	mov r1,3		;2
grybr2:
	djnz r1,grybr2	;2
donext:	
	inc r0			;1\ next display column
	inc r0			;1  adjust for djnz
	djnz r0,doledsb	;2/4
dsp_end:	
	mov a,r2
	jnz dsp_ret
dsp_off:
	mov	r1,IRAM_DSPMODE 
	mov a,@r1
	anl a,$9F		;clear bleeding and scroll
	mov	@r1,a
dsp_ret:
    mov r0,1		;blank vram addr
	call ledbar		;do blank line
	anl p2,0		;enable display blanking
	sel rb1
;	ret
	
.ifdef CODERED_ASM	
	call M_PLAY
.endif	
	;jmp		AV_TRIGGER
	
;---------------------------------------
;Check out of demo part frames
;---------------------------------------

;check if frame counter overflowed. 
;Meaning the demo part is out of frames
;and should end immediately

;entry:
;	none
;exit:
;	NC frame counter overflowed
;    C frame counter is still counting

;changes:
;	a,r0

AV_TESTTRIGGER:
	mov	r0,IRAM_FRAMES_MSB
	mov	a,@r0
	rlc a
	ret	
	
;---------------------------------------
;Test demo part frames to go (byte)
;---------------------------------------

;entry:
;	r2 = positive number of frames to test against

;exit:
;	NC more then amount of frames to go
;    C exact or less frames to go

;changes:
;	a,r0,r1

AV_TESTTRIGGER1:
	mov	r1,0
;---------------------------------------
;Test demo part frames to go (word)
;---------------------------------------

;entry:
;	r2 = positive number of frames to test against(LSB)
;	r1 = positive number of frames to test against(MSB)

;exit:
;	NC more then amount of frames to go
;    C exact or less frames to go

;changes:
;	a,r0

AV_TESTTRIGGER2:
	mov	r0,IRAM_FRAMES
	mov	a,@r0
	add	a,r2	
	inc	r0	
	mov	a,r1
	addc a,@r0	;CY > r1.r2 frames or less to go
	ret
	
;do sound
;---------------------------------------
;entry:
;	a = Sound command

;exit:
;	none

;uses:
;	a, rb0(r1,r2,r3)

AV_SOUND:
	sel rb0
	mov r1, a		;save sound command
	movx a,@r1		;save old vram data
	mov r3,a
	mov a, $C0
	outl p2,a		; Control sound
	clr a					
	movx @r1,a
	movx a,@r1		;reset COP411A
	mov r2,$14
	djnz r2,$		;delay 40 cycles
	inc a			;bit 0=1 
	movx @r1, a			
	movx a,@r1		;release COP411A from reset state
	mov a,r1
	outl p2,a		; Write high nibble
	mov r2,$26
	djnz r2,$
	swap a
	outl p2,a		; Write low nibble
	mov r2,$21
	djnz r2,$
	anl p2,$00		; deactivate COP411 reset logic and clear ledbar
	mov a,r3
	movx @r1,a		;restore corrupted vram data
	sel	rb1
	ret

;m player sound fix
;---------------------------------------
	
;skips one mplayer frame for code that 
;takes more time then a single frame to complete
	
;AV_PLAYFIX:
;	mov	r0, M_PLAY_CTR		;m player heartbeat
;	mov	a,@r0
;	jb0 playfix				;save to skip the non play frame
;	call AV_SHOW			;sound frame must be played 
;	jmp  AV_PLAYFIX		;try again
;	
;playfix:	
;	mov	r0, M_PLAY_CTR		;skip even sound frame to keep audio in line
;	jmp M_PLAY_END			;for code that takes more then 1 fram
	
.endif ;MEGABIOS_ASM	
;===============================================================================
