	; Adventure Vision Demo
	; * Mega Logo Zoomer 
	; by sy2002, Mar 24 2013
	
	; MEGA-ZOOMER CORE FUNCTIONS

;------------------------------------------------------------
; Display the logo at (x|y) = (r3|r4)
; using the zoom-level (zoom-x|zoom-y) = (r5|r6) as index
; to the x/y-ratio lookup-tables
; r3 & r4 must be chosen to fit the logo w/o clipping
; destroys all registers and uses local variables
;------------------------------------------------------------

; "local" variables, only used by this function
; IMPORTANT:	The code relies on the the byte ordering of the
;				HI/LO bytes: HI first, then in the subsequent byte LO
.equ LMZ_ORG_X, IRAM_MZ_LOCAL1		; original x position
.equ LMZ_ORG_Y, IRAM_MZ_LOCAL2
.equ LMZ_ZOOM_X, IRAM_MZ_LOCAL3		; absolute width of target zoom size
.equ LMZ_ZOOM_Y, IRAM_MZ_LOCAL4		; absolute height of target zoom size
.equ LMZ_RATIO_X_HI, IRAM_MZ_LOCAL5	; hi-byte of current x-zoom level
.equ LMZ_RATIO_X_LO, IRAM_MZ_LOCAL6	; lo-byte of current x-zoom level
.equ LMZ_RATIO_Y_HI, IRAM_MZ_LOCAL7	; hi-byte of current y-zoom level
.equ LMZ_RATIO_Y_LO, IRAM_MZ_LOCAL8	; lo-byte of current y-zoom level
.equ LMZ_RAT_HI, IRAM_MZ_LOCAL9		; hi-byte of variable "rat" in algorithm.c
.equ LMZ_RAT_LO, IRAM_MZ_LOCAL10	; lo-byte of variable "rat" in algorithm.c

displayzoomedlogo:
	; remember original (x|y)-pos, as r3, r4 are used
	mov r0, LMZ_ORG_X
	mov a, r3
	mov @r0, a
	inc r0							; works as long as LMZ_ORG_Y is LMZ_ORG_X + 1
	;mov r0, LMZ_ORG_Y
	mov a, r4
	mov @r0, a
	
	; read the absolute zoomed target sizes (new width & height) as well
	; as the width/height ratios (16-bit) from the LUT and store to local vars
	mov a, CMZ_RATIOS_LUT_BANK		; switch to LUT bank
	outl p1, a
	mov a, r5						; pointer to x-lut using r5 as index
	add a, r5						; ... which needs to be multiplied by 3
	add a, r5						; (and tripling the "add" line costs as much as a loop)
	mov r0, a
	jmp pixelmapping_data_end

	; inline lookup-table and align it with $E00, i.e. so that movp a, @a can work
pixelmapping_data:
	.db %01111111
	.db %10111111
	.db %11011111
	.db %11101111
	.db %11110111
	.db %11111011
	.db %11111101
	.db %11111110
pixelmapping_data_end:

	movx a, @r0	
	mov r1, LMZ_ZOOM_X				; store absolute width of target zoom size
	mov @r1, a
	inc r0
	movx a, @r0
	mov r1, LMZ_RATIO_X_HI			; store hi-byte of width-ratio
	mov @r1, a
	inc r0
	movx a, @r0
	inc r1							; works as long as LMZ_RATIO_X_LO is LMZ_RATIO_X_HI + 1
	;mov r1, LMZ_RATIO_X_LO			; store lo-byte of width-ratio
	mov @r1, a
	mov a, CMZ_YRATIOS_LUT_OFFS		; pointer to y-lut according to ...
	add a, r6						; ... the index given as parameter r6 ...
	add a, r6						; ... which needs to be multiplied by 3
	add a, r6						; (and tripling the "add" line costs as much as a loop)
	mov r0, a
	movx a, @r0
	mov r1, LMZ_ZOOM_Y				; store absolute height of target zoom size
	mov @r1, a
	inc r0
	movx a, @r0
	mov r1, LMZ_RATIO_Y_HI			; store hi-byte of height-ratio
	mov @r1, a
	inc r0
	movx a, @r0
	inc r1							; works as long as LMZ_RATIO_Y_LO is LMZ_RATIO_Y_HI + 1
	;mov r1, LMZ_RATIO_Y_LO			; store lo-byte of width-ratio
	mov @r1, a

	; rb0 is used for this function, rb1 is used while doing putpixel
	sel rb0							; rb0 for this function, rb1 for subroutines

	; initialize "y_ratio_mult" as described in algorithm.c
	clr a
	mov r2, a						; hi-byte of "y_ratio_mult"
	mov r3, a						; lo-byte of "y_ratio_mult"
	
	; "i" aka y-position is set to the original y
	mov r0, LMZ_ORG_Y
	mov a, @r0
	mov r4, a

dzl_next_line:

	; "j" aka x-position is reset to the original x
	mov r0, LMZ_ORG_X
	mov a, @r0
	mov r5, a

	; find the base pointer within the pixels array by multiplying the
	; original width "w1" by "y_ratio_mult >> 8" and then adding the CMZ_RAMCACHE_OFFS
	; "y_ratio_mult >> 8" is simply r2, because ">> 8" equals taking the hi-byte
	; currently assuming a fixed logo size: w1 == ARTWORK_DX
	mov a, r2
	jz dzl_nl_mul_end
	sel rb1
	mov r1, a
	clr a
dzl_nl_mul:
	add a, ARTWORK_DX
	djnz r1, dzl_nl_mul
	sel rb0
dzl_nl_mul_end:
	; (@@1) uncomment the following line, if CMZ_RAMCACHE_OFFS is not zero
	; add a, CMZ_RAMCACHE_OFFS
	
	; use r6 as the "p" pointer that is used as a pixel offset
	mov r6, a
		
	; r7 is the loop variable for the "j loop"
	mov r0, LMZ_ZOOM_X
	mov a, @r0
	mov r7, a
	
	; initialize "rat" from algorithm.c, i.e. the x-ratio "adder"
	; which is a replacement for another multiplication
	clr a
	mov r0, LMZ_RAT_HI
	mov @r0, a
	inc r0							; works as long as LMZ_RAT_LO = LMZ_RAT_HI + 1
	;mov r0, LMZ_RAT_LO
	mov @r0, a

	; switch to RAM cache bank for being able to read the current source pixel
	mov a, CMZ_RAMCACHE_BANK		
	outl p1, a	

dzl_next_pixel_in_current_line:

	; find the source byte inside the pixel pattern
	mov r0, LMZ_RAT_HI				; taking the hi-byte equals "rat >> 8"
	mov a, @r0
	add a, r6						; equals "p + x2"
	
	; read the source byte 
	mov r0, a
	movx a, @r0
	jz dzl_skip_pixel
	
	; switch to VRAM since putpixel expects an active VRAM bank to write to
	orl p1, XVRAM3	
		
	; putpixel at (x|y) = (r0|r1), if the source byte is 1
	mov a, r5
	sel rb1
	mov r0, a
	sel rb0
	mov a, r4
	sel rb1
	mov r1, a

	; Put a pixel to VRAM
	; r0 = x: 0..49; r1 = y: 0..39
	; (destroys a, r0..r2)
	; Inlined subroutine

	; handle clipping	
	mov a, r0
	orl a, r1
	jb7 putpixel_ret	; x < 0 or y < 0		
	mov a, -1 - 49
	add a, r0
	jc putpixel_ret		; x > 49
	mov a, -1 - 39
	add a, r1
	jc putpixel_ret		; y > 39
	
	; x-pos in memory = 6 + (x * 5), i.e. 6 + (x << 2) + x
	mov a, r0
	rl a
	rl a
	add a, r0
	add a, 6
	mov r0, a	; r0 now stores the offset to the VRAM

	; y-byte-pos in memory = (current offset aka r0) + 4 - (y / 8)
	; because we have 5 one byte registers, each responsible for 8 leds
	; so we count the offset from 0 .. 4 (therefore + 4 - ...)
	; for speeding up the operation:
	; (y / 8) integer divide equals y >> 3
	; y >> 3 equals((y << 1) & %11110000) and then swap hi/lo nibbles
	mov a, r1			; integer divide y / 8, i.e. r1 / 8 and store to r0
	rl a
	swap a
	cpl a				; a = -1 - a
	anl a, %00000111	; 0..4 > 7..3
	add a, -3			; 4..0
	add a, r0
	mov r0, a
	
	; a will contain y modulo 8, which reflects the bit position of the pixel
	; within the byte referenced by r0 (aka x|y-byte pos)
	; y modulo 8 equals (y & %00000111) in this case, because the lower 3 bytes
	; are the ones, that are kind of "shifted right out of the byte" in above
	; mentioned division that is emulated by y >> 3 (i.e. swap((y << 1) & %11110000))
	mov a, r1
	anl a, %00000111

	; virtually shift right a "0" bit through a %11111111 field, because "0" = pixel on
	; use a (aka y mod 8) as pointer to the pixelmapping table
	movp a, @a
	mov r2, a
	
	movx a, @r0		; get existing byte in VRAM and set the new bit
	anl a, r2
	movx @r0, a

putpixel_ret:
		
	; switch to RAM cache bank for being able to read the current source pixel
	mov a, CMZ_RAMCACHE_BANK		
	outl p1, a	

dzl_skip_pixel:	
	; 16bit addition "rat += x_ratio;"
	; i.e. LMZ_RAT_HI.LMZ_RAT_LO += LMZ_RATIO_X_HI.LMZ_RATIO_X_LO
	sel rb1
	mov	r0, LMZ_RAT_LO
	mov	r1, LMZ_RATIO_X_LO
	mov	a, @r0
	add	a, @r1						; add the LO bytes and set carry
	mov	@r0, a						; store LO byte
	dec	r0							; switch to the HI bytes ...
	dec	r1							; ... expects HI/LO fixed byte order
	mov	a, @r0
	addc a, @r1						; add the HI bytes and carry from LO
	mov	@r0, a						; store HI byte
	sel rb0
	
	; x-loop (aka width-loop ala "for j loop")
	inc r5							; j++ aka next x-position
	djnz r7, dzl_next_pixel_in_current_line
	
	; 16 bit addition "y_ratio_mult += y_ratio;"
	; i.e. r2.r3 += LMZ_RATIO_Y_HI.LMZ_RATIO_Y_LO
	sel rb1
	mov	r0, LMZ_RATIO_Y_LO
	mov	a, @r0
	sel	rb0
	add	a, r3						; r3 = lo-byte of "y_ratio_mult"
	mov	r3, a
	sel	rb1
	dec	r0							; switch to HI byte, expects HI/LO fixed byte order
	mov	a, @r0
	sel	rb0
	addc a, r2						; r2 = hi-byte of "y_ratio_mult"
	mov	r2, a
		
	; y-loop (aka height-loop aka "for i loop")
	mov r0, LMZ_ZOOM_Y
	mov a, @r0
	dec a
	mov @r0, a
	inc r4							; i++ aka next y-position
	jnz dzl_next_line
	
	sel rb1	
	ret
	
	
;------------------------------------------------------------
; Unpack the logo to EXT RAM for achieving a faster
; pixel access for the nearest neighbour algorithm
; CMZ_ constants are defining the bank and the location
; oversimplified code that can only handle a max height of
; the logo of 8 pixels
; expects the LO byte of the logo offset in artwork.asm in r7
; destroys a, r0, r2, r3, r4, r5
;------------------------------------------------------------

; hardcoded start position of the y-scanner
; the upper 3 bit are padding since the logo is only
; 5 pixels high; for other images, this needs to be adjusted
.equ MZC_SCANPATTERN_START, %00010000

mz_unpacklogo:
	; switch to target EXT ram bank and initialize r0 as offset counter
	mov a, CMZ_RAMCACHE_BANK
	outl p1, a
	
	clr a
	mov r0, a				; CMZ_RAMCACHE_OFFS is now assumed to be hardcoded 0
	
	; initialize registers for accessing the logo
	mov r2, a				; r2 as pointer to the current byte
	mov r3, ARTWORK_SIZE	; r3 as size counter that can be djnz'ed
	mov r4, ARTWORK_DY

	; this bit will be rotated to the right to test set pixels
	; the highest bit equals the lowest y-coordinate
	mov r5, MZC_SCANPATTERN_START
	clr c					; the carry bit will be used as overflow flag
	
unpack_browse_x:
	; clear memory
	; the layout is dx * dy, 1 byte per pixel; 1 = set, 0 = not set
	; (that means we use the intuitive logic here, versus the AV logic)
	clr a
	movx @r0, a

	; get next byte (aka x-position) from megalogo
	mov a, r2
	add a, r7
	call getlogo

	; check if a pixel is set (i.e. bit = 0 due to AV logic, i.e. reverse storage)
	anl a, r5
	jnz unpack_nextcolumn
	inc a					; a is guaranteed to be "1" here
	movx @r0, a				; set pixel at current (x|y) aka (r0)
	
unpack_nextcolumn:
	; increase the RAM cache storage offset	as we walk like x++, then y++...
	inc r0					

	; ...we run through the logo multiple times due to that reason
	inc r2
	
	djnz r3, unpack_browse_x
	
	; when we arrive here, a x-line has be done, so we need to go to
	; the next y-position, meaning the next higher bit
	mov r2, 0				; reset the megalogo byte pointer
	mov r3, ARTWORK_SIZE	; reset the djnz'ed size counter

	mov a, r5
	rrc a					; rotate the "scanner" bit to the next y-pos
	mov r5, a				; remember rotated scanner bit
	jnc unpack_browse_x		; wrap-around (carry) == we've reached the final y-pos
unpack_done:	
	ret


;------------------------------------------------------------
; Create lookup tables for width and height ratios
; and for the zoomslide (y-interpolation)
; for avoiding the need of realtime 16bit divisions inside
; the realtime code in displayzoomedlogo
; CMZ_ constants are defining the bank and the location
; uses both register banks
; destroys accu and all registers
;------------------------------------------------------------

mz_create_luts:

	sel rb0							; register bank 0 for the loop

	; switch to LUT EXT ram bank
	mov a, CMZ_RATIOS_LUT_BANK
	outl p1, a
	
	; X-RATIOS LUT: calculate and store it
	
	call mz_create_luts_two_loc		; the call is doing the two commented out
									; lines of code to free more space in this code page and
									; to fill those 5 unused bytes in page $A
	;mov r0, CMZ_XRATIOS_LUT_OFFS	
	;mov r1, CMZ_XRATIOS_LUT_START

	mov r3, CMZ_XRATIOS_LUT_END		
	mov r2, ARTWORK_DX
	call create_luts_do_ratios
	
	; Y-RATIOS LUT: calculate and store it
	mov r0, CMZ_YRATIOS_LUT_OFFS
	mov r1, CMZ_YRATIOS_LUT_START
	mov r3, CMZ_YRATIOS_LUT_END		
	mov r2, ARTWORK_DY
	call create_luts_do_ratios

	; switch to zoomslide EXT ram bank and initialize r0 as loopvar
	mov a, CMZ_ZOOMSLIDE_BANK
	outl p1, a
	mov r0, CMZ_ZOOMSLIDE_OFFS
	
	; create interpolation table for y (zoomslide)
	; WARNING: this code only works for modulo == 0
	clr a
	mov r3, a
	mov r2, CMZ_XRATIOS_SIZE	
	mov r5, a
	mov r4, CMZ_YRATIOS_SIZE
	call divide
	mov a, r2						; r2 contains the result (lo-byte)
	mov r1, a						; use r1 as "inner loop" var for interpolation
	
	mov r3, 0						; r3 is a pointer to the y_scale LUT
	mov r4, CMZ_YRATIOS_SIZE		; r4 is the loopvar for the table ("outer loop")
	
create_luts_yslide:
	mov a, r3
	movx @r0, a						; store pointer (current interpolation level)
	inc r0
	djnz r1, create_luts_yslide
	
	mov a, r2						; r2 still contains the division result from above
	mov r1, a						; reset r1
	inc r3							; next interpolation level (aka ptr to y_scale LUT)
	djnz r4, create_luts_yslide
		
	ret
	
	; Subroutine of create_luts to actually do the job
	; r0 = target byte offset, r1 = LUT start value (that iterates)
	; r2 = divisor (i.e. w2 or h2), r3 = LUT end value
	
create_luts_do_ratios:

	mov a, r1						; LUT: store width
	movx @r0, a
	inc r0
		
	; "x|y_ratio" = r4.r5 = (int)((dividend << 8) / divisor) + 1
	; for more details, see algorithm.c
	; dividend = (x:w1) or (y:h1) depending if we're doing "x_ratio" or "y_ratio"
	; divisor = (x:w2) or (y:h2), dito.
	; dividend << 8 just means to put it to r2; divisor == r1
	mov a, r2
	sel rb1							; register bank 1 for the division
	mov r3, a
	mov r2, 0
	sel rb0
	mov a, r1						; divisor == r1
	sel rb1
	mov r5, 0
	mov r4, a	
	call divide						; 16bit division: r3.r2 = r3.r2 / r5.r4		
	mov a, r3
	sel rb0
	mov r4, a						; r4: result hi-byte
	sel rb1
	mov a, r2
	sel rb0
	inc a							; 16bit add r4.r5 = r4.r5 + $00.$01
	mov r5,a
	jnz $+3
	inc r4
	
	; LUT: store "x|y_ratio" in the order hi/lo
	mov a, r4
	movx @r0, a
	inc r0
	mov a, r5
	movx @r0, a
	inc r0
	
	inc r1							; next "x|y_ratio"
	
	; if (r3 - r1 < 0), then we're done
	; "-1" and not "0", because we want the full range just lika a do/while instead of while/do	
	mov a, r3
	cpl a
	add a, r1						; since we omit the "cpl" we have inverse logic, ...
	jb7 create_luts_do_ratios		; ... i.e. bit7 set = we are > 0
	
create_luts_do_ratios_ret:
	ret


;--------------------------------------------------------------------
; MAIN ROUTINE: Run the effect
; fill the following variables before calling:
; IRAM_MZ_XPOS: starting x-position
; IRAM_MZ_YPOS: starting y-position
; MZ_ALLOW_XY: bit7 = x-movement, bit0 = y-movement
;              bit cleared = allow, bit set = prevent
; MZ_ALLOW_DX: allow x-zooming: 0 = true, >0 = fixed value
; MZ_ALLOW_DY: allow y-zooming: 0 = true, >0 = fixed value
; MZ_TOGGLE_DELAY: delay 2 seconds after toggling direction
; MZ_STARS: do stars: 1 = true, 0 = false
; MZ_SHOW_CR: 0 = at the end of the sequence: switch to CODE RED
;			  only relevant in sequence mode, i.e when MZ_STARS = 1
; destroys accu and all registers
;--------------------------------------------------------------------

.equ MZ_ALLOW_XY, IRAM_MZ_LOCAL11
.equ MZ_ALLOW_DX, IRAM_MZ_LOCAL12
.equ MZ_ALLOW_DY, IRAM_MZ_LOCAL13
.equ MZ_TOGGLE_DELAY, IRAM_MZ_LOCAL14

.equ MZ_STARS, IRAM_MZ_LOCAL15
.equ MZ_SHOW_CR, IRAM_MZ_LOCAL16

mz_runeffect:

	; we have one LUT for the circle's x and y coordinates,
	; but different starting points for x and y for a good movement
	mov r0, IRAM_MZ_CIRCLE_X_IDX
	mov @r0, CMZ_CIRCLE_START_X
	inc r0								; IRAM_MZ_CIRCLE_Y_IDX 
	mov @r0, CMZ_CIRCLE_START_Y
		
	; reset the zoomslide and the zoomslide counter
	mov r0, IRAM_MZ_ZOOMSLIDE
	mov @r0, 0
	inc r0								; IRAM_MZ_ZOOMSLIDE_CNTR
	mov @r0, CMZ_YRATIOS_SIZE
	inc r0								; IRAM_MZ_ZOOMSLIDE_DIR 
	mov @r0, -1
	
displayloop:

	; clear the screen
	sel mb0
	call AV_CLS_VRAM3
	sel mb1
	
	; draw starfield, if appropriate
	; important: the starfield LUTs need to be initialized in XRAM1
	; before draw_dndn_stars can be called; kills all registers
	mov r0, MZ_STARS
	mov a, @r0
	jz dl_no_stars
	
	; handle sequences 1..3 defined by storyboard.txt's Scene #6
	jmp mz_scene_06_sequencer
	
	; handle sequence 4, due to ROM/page restrictions in seq.asm
mz_scene_6_seq4:
	mov a, @r0						; r0 = MZ_SHOW_CR
	jnz mz_scene_6_seq4_contd		; if @r0 = 0: use coderedlogo
	inc @r0							; but only once, not each frame
	mov r7, <coderedlogo
	call mz_unpacklogo
	
mz_scene_6_seq4_contd:
	mov a, STARS_ROTATE_LEFT		; stars start to rotate left
	
dl_show_stars:
	call draw_dndn_stars
	
dl_no_stars:
	; displaylogo expects (x|y) = (r3|r4)
	; therefore: get current position LUT entry and add it
	; to the initial positions and store it in r3 and r4
	; check if the movement is allowed at all
	mov r0, IRAM_MZ_XPOS
	mov a, @r0
	mov r3, a						; r3 = current x-pos for displaylogo
	mov r0, MZ_ALLOW_XY
	mov a, @r0
	jb7 dl_x_done
	mov r0, IRAM_MZ_CIRCLE_X_IDX	; x-position / x-index inside lut
	mov a, @r0 
	call circle_lut
	add a, r3						; add the LUT result to the initial x-pos
	mov r3, a
dl_x_done:

	mov r0, IRAM_MZ_YPOS
	mov a, @r0
	mov r4, a						; r4 = current y-pos for displaylogo
	mov r0, MZ_ALLOW_XY
	mov a, @r0
	jb0 dl_y_done
	mov r0, IRAM_MZ_CIRCLE_Y_IDX
	mov a, @r0
	call circle_lut
	add a, r4						; add the LUT result to the initial y-pos				
	mov r4, a
dl_y_done:
				
	; display the zooming and moving logo
	mov r0, IRAM_MZ_ZOOMSLIDE
	mov a, @r0
	mov r5, a					; x-zoom level
	mov a, CMZ_ZOOMSLIDE_BANK	; switch to zoomslide bank
	outl p1, a					
	mov a, CMZ_ZOOMSLIDE_OFFS	; offset to interpolation table
	add a, r5					; offset within interpolation table
	mov r0, a
	movx a, @r0					; read interpolation table for y
	mov r6, a					; y-zoom level

	; (x|y) = (r3|r4) and zoomlevel (zx|zy) = (r5|r6)
	mov r0, MZ_ALLOW_DX
	mov a, @r0
	jz $ + 3
	mov r5, a
	inc r0
	mov a, @r0
	jz $ + 3
	mov r6, a
	call displayzoomedlogo		; displaying destroys all registers and
								; uses internal variables at IRAM_MZ_LOCALx
															
	; sync mirror and draw screen via LEDs
	mov a, XVRAM3
	outl p1, a
	sel mb0
	call AV_SHOW
	sel mb1
	jnc dl_inc_circle_idx_ret

	; increment or decrement zoomslide
	mov r0, IRAM_MZ_ZOOMSLIDE_DIR	; direction of movement
	mov a, @r0
	mov r1, a
	mov r0, IRAM_MZ_ZOOMSLIDE
	mov a, @r0
	cpl a
	add a, r1						; -(-1) leads to "+1", i.e. -1 = zoom bigger
	cpl a
	mov @r0, a

	; min reached
	inc a							; compensate one decrement too much from above
	jz dl_slide_flip_p1

	; max reached?
	cpl a
	add a, CMZ_XRATIOS_SIZE
	cpl a
	jz dl_slide_flip
	
	jmp dl_circle_lut
	
dl_slide_flip_p1:
	mov r0, IRAM_MZ_ZOOMSLIDE
	inc @r0
	
dl_slide_flip:
	mov r0, MZ_TOGGLE_DELAY
	mov a, @r0
	jz dl_nodelay

	mov r4, a		; r0 .. r3 destroyed by AV_SHOW
	sel mb0
	
dl_delay_loop:
	call AV_SHOW
	djnz r4, dl_delay_loop
	sel mb1
	
dl_nodelay:
	; flip zoomslide counter
	mov r0, IRAM_MZ_ZOOMSLIDE_CNTR
	mov @r0, CMZ_YRATIOS_SIZE
	inc r0							; IRAM_MZ_ZOOMSLIDE_DIR
	mov a, @r0
	inc a
	jz dl_slide_switch_out	
	
	mov a, -1
	
	; skip the following inc, so that the "-1" is stored in @r0
	.db $BE							; thx to JAC!'s magic
									; equivalent to jmp dl_slide_storeit
									; but uses the opcode of mov r6, const
									; for "abusing" the following inc command
									; as the "const" and therefore ignoring it
									; and therefore squeezing out one more byte
									
dl_slide_switch_out:
	inc a

dl_slide_storeit:
	mov @r0, a
	
dl_circle_lut:
	mov r0, IRAM_MZ_CIRCLE_X_IDX
	call dl_inc_circle_idx
	
	mov r0, IRAM_MZ_CIRCLE_Y_IDX
	call dl_inc_circle_idx

	;loop the demopart
	jmp displayloop
	
	; subroutine to handle the two different circle LUT index counters
dl_inc_circle_idx:
	mov a, @r0						; increment index
	inc a
	mov @r0, a
	
	add a, -CIRCLE_SIZE				; index  == CIRCLE_SIZE? 
	jnz dl_inc_circle_idx_ret		; then (since count starts from zero) we're done
	
	clr a							; else reset the index
	mov @r0, a
	
dl_inc_circle_idx_ret:
	ret